/**
 * JASMINe
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.rules;

import java.io.File;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.drools.KnowledgeBase;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderConfiguration;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.conf.DumpDirOption;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;
import org.ow2.jasmine.rules.api.DroolsWorkingMemoryState;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemory;

import static org.drools.builder.KnowledgeBuilderFactory.newKnowledgeBuilderConfiguration;
import static org.ow2.jasmine.rules.api.DroolsWorkingMemoryState.*;

public abstract class AbstractDroolsWorkingMemory implements
        IDroolsWorkingMemory {

    /**
     * Logger.
     */
    private static Logger logger = null;

    /**
     * Thread to fire rules.
     */
    protected Thread thread = null;

    /**
     * The managed knowledge session.
     */
    protected volatile StatefulKnowledgeSession knowledgeSession = null;

    /**
     * {@link ClassLoader} to build rules.
     */
    protected ClassLoader classLoader;

    /**
     * The KnowledgeBase
     */
    protected KnowledgeBase knowledgeBase;

    /**
     * The state of the working memory
     */
    protected volatile DroolsWorkingMemoryState state;

    /**
     * The configuration used to create package builders
     */
    protected KnowledgeBuilderConfiguration knowledgeBuilderConf;

    /**
     * Rules fire period
     */
	private static int SLEEP_TIME_BETWEEN_FIRE = 500;

    /**
     * Only used in synchronized blocks
     */
    private final Object monitor = new Object();

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    protected AbstractDroolsWorkingMemory(final Logger logger) {

        AbstractDroolsWorkingMemory.logger = logger;
        state = IDLE;
    }

    /**
     * @param classLoader
     */
    protected void initKnowledgeBuilderConfiguration(final ClassLoader classLoader) {

        Properties props = new Properties();
        props.setProperty("drools.dialect.java.compiler", "JANINO");
        this.knowledgeBuilderConf = newKnowledgeBuilderConfiguration(props, classLoader);

        // In development, dump drools generated files
        if (Boolean.getBoolean("jonas.developer")) {
            File dumpDir = new File(System.getProperty("java.io.tmpdir"), "drools_dump");
            boolean exists = dumpDir.exists();
            if (!exists) {
                exists = dumpDir.mkdir();
            }
            if (exists && dumpDir.isDirectory()) {
                debug("WorkingMemory[<name>] Drools dump directory set to {0}", dumpDir);
                knowledgeBuilderConf.setOption(DumpDirOption.get(dumpDir));
            }
        }
    }


    // ------------------------------------------------------------------------
    // Implementation of the IDroolsWorkingMemory interface
    // ------------------------------------------------------------------------

    /**
     * Stop the WorkingMemory, with the state STOPPED.
     * @see {@link AbstractDroolsWorkingMemory#stop(DroolsWorkingMemoryState)}
     */
    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.api.IDroolsWorkingMemory#stop()
     */
    public void stop() {
        stop(STOPPED);
    }

    /**
     * Stop the working memory with a given state (allowed states are
     * STOPPED and IDLE)
     */
    public void stop(DroolsWorkingMemoryState state) {
    	synchronized (monitor) {
    		if (knowledgeSession != null) {
    			knowledgeSession.dispose();
    		}
    	}
    	knowledgeSession = null;

        // As the session is null, the executor should stop by itself
        // to enforce this behavior, wait for the thread to stop nicely
        if (thread != null) {
            try {
                // Wait for a loop time for the Executor (give it at
                // least 1 chance to read the field value)
                thread.join(SLEEP_TIME_BETWEEN_FIRE);
            } catch (InterruptedException e) {
                // Someone interrupted this Thread
                // no problem for us as we want it to be stopped
                // so continue the execution ...
            }

            // Force interrupt if Thread is still alive
            if (thread.isAlive()) {
                thread.interrupt();
            }
            thread = null;
        }

        DroolsWorkingMemoryState newState;
        if( state == IDLE || state == STOPPED ) {
            newState = state;
        } else {
            newState = STOPPED;
        }

        debug("WorkingMemory[<name>] Stopped. State changed from {0} to {1}", this.state, newState);
        this.state = newState;
    }

    /**
     * Insert a fact in the WorkingMemory
     * @param fact the fact object to insert
     * @return the corresponding FactHandle
     */
    public FactHandle insert(final Object fact) {
        if (isReady()) {
        	synchronized (monitor) {
                debug("WorkingMemory[<name>] Inserting fact into session.");
        		return knowledgeSession.insert(fact);
        	}
        } else {
            debug("WorkingMemory[<name>] Cannot insert fact into session: not RUNNING or no session.");
            return null;
        }
    }

    /**
     * update a fact in the WorkingMemory
     * @param handle the fact handle object to update
     * @param object the new value
     */
    public void update(FactHandle handle, Object object) {
        if (isReady()) {
        	synchronized (monitor) {
                debug("WorkingMemory[<name>] Updating fact into session.");
        		knowledgeSession.update(handle, object);
        	}
        } else {
            debug("WorkingMemory[<name>] Cannot update fact into session: not RUNNING or no session.");
        }
    }

    /**
     * @return {@code true} if the KnowledgeSession is ready to be used.
     */
    private boolean isReady() {
        return (state != STOPPED) && (knowledgeSession != null);
    }

    // ------------------------------------------------------------------------
    // Protected methods
    // ------------------------------------------------------------------------

    /**
     * Create a new working memory for a given rule base.
     */
    protected void createWorkingMemory() {
        debug("WorkingMemory[<name>] Create the KnowledgeSession");
        knowledgeSession = knowledgeBase.newStatefulKnowledgeSession();
    }

    /**
     * Start the rule execution process
     */
    protected void startRuleExecutor() {
        state = RUNNING;
        debug("WorkingMemory[<name>] State is now {0}", this.state);
        thread = new Thread(new RuleExecutor(), "Rule-Executor");
        thread.start();
        debug("WorkingMemory[<name>] RuleExecutor thread has been started");
    }

    /**
     * Periodically fire rules.
     */
    private class RuleExecutor implements Runnable {

        /**
         * Execute the rules after the timeout
         */
        public void run() {
        	while (isReady()) {
        		synchronized (monitor) {
        			try {
        				int fired = knowledgeSession.fireAllRules();
                        debug("WorkingMemory[<name>] # of fired rules: {0}", fired);
        			} catch (Exception e) {
        		        logger.log(Level.SEVERE, "Exception during fireAllRules() - " + e.getMessage());
        			}
        		}
    			try {
    				Thread.sleep(SLEEP_TIME_BETWEEN_FIRE);
    			} catch (InterruptedException e) {
                    // Can be interrupted when memory is stopped: no worries
    		        logger.throwing(getClass().getSimpleName(), "run()", e);
    			}
        	}
            debug("WorkingMemory[<name>] Cleanly exiting the RuleExecutor thread");
        }
    }

    /**
     * Get the KnowledgeBase
     * @return the KnowledgeBase
     */
    public KnowledgeBase getKnowledgeBase() {
        return knowledgeBase;
    }

    /**
     * Get the current state of the working memory
     * @return the state
     * @see {@link org.ow2.jasmine.rules.api.DroolsWorkingMemoryState}
     */
    public DroolsWorkingMemoryState getState() {
        return state;
    }

    /**
     * Get a KnowledgeBuilder to compile a rule package externally
     * @return an initialized KnowledgeBuilder
     */
    public KnowledgeBuilder newKnowledgeBuilder() {
        return KnowledgeBuilderFactory.newKnowledgeBuilder(knowledgeBuilderConf);
    }

    /**
     * Log a parametrized message a FINE level.
     * @param message log message
     * @param vars message's parameters
     */
    protected void debug(String message, Object... vars) {
        logger.log(Level.FINE, message, vars);
    }
}
