/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import java.util.HashMap;
import java.util.Map;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

/**
 * Activator that registers a package adder for this bundle, this defining rules.
 * Note that rules have to belong to a same package.
 * This class also provide the working memory to allow bundles adding some globals.
 * @author Loris Bouzonnet
 */
public abstract class RulesActivator implements BundleActivator {

    /**
     * A service registration for the package adder of this bundle.
     */
    private ServiceRegistration packageAdderRegistration;

    private final Map<String, Object> globals = new HashMap<String, Object>();

    /**
     * The targeted working memory name
     * @see OSGiDroolsWorkingMemoryPool
     */
    protected String workingMemoryName = null;

    
    public final void start(final BundleContext context) throws Exception {

        // Perform the real start of the bundle
        doStart(context);

        // Register a package adder for this bundle
        packageAdderRegistration = (workingMemoryName == null)?
            context.registerService(
            		IPackageAdder.class.getName(),
                    new BundlePackageAdder(context.getBundle(), globals),
                    null):
            context.registerService(
            		IPackageAdder.class.getName(),
                    new BundlePackageAdder(context.getBundle(), globals, workingMemoryName),
                    null);
    }

    public final void stop(final BundleContext context) throws Exception {

        // Unregister the package adder
        if(packageAdderRegistration != null) {
            packageAdderRegistration.unregister();
            packageAdderRegistration = null;
        }

        // Perform the real stop of the bundle
        doStop(context);
    }

    /**
     * Perform the real start of the bundle.
     * @param context bundle context
     * @throws Exception if the bundle cannot be started
     */
    protected abstract void doStart(BundleContext context) throws Exception;

    /**
     * Perform the real stop of the bundle.
     * @param context bundle context
     * @throws Exception if the bundle cannot be stopped
     */
    protected abstract void doStop(BundleContext context) throws Exception;

    protected void setGlobal(String name, Object value) {
        globals.put(name, value);
    }

}
