/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.manager.service;

import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.drools.KnowledgeBase;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.ResourceType;
import org.drools.definition.KnowledgePackage;
import org.drools.io.ResourceFactory;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jasmine.rules.api.DroolsWorkingMemoryState;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemory;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemoryPool;
import org.ow2.jasmine.rules.manager.model.AbstractRulePackage;
import org.ow2.jasmine.rules.manager.model.IRulePackage;
import org.ow2.jasmine.rules.manager.model.textual.TextualRulePackage;
import org.ow2.jasmine.rules.manager.model.visual.LHSCodeSnippet;
import org.ow2.jasmine.rules.manager.model.visual.RHSCodeSnippet;
import org.ow2.jasmine.rules.manager.model.visual.Rule;
import org.ow2.jasmine.rules.manager.model.visual.VisualRulePackage;
import org.ow2.jasmine.rules.manager.persistence.RulePackageSLBRemote;
import org.ow2.jasmine.rules.manager.util.jndilocator.ServiceLocator;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;
import org.ow2.jasmine.rules.osgi.impl.DrlPackageAdder;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;


/**
 * Manage a given Drools working memory
 *
 * @author Julien Nicoulaud
 * @see {@link IDroolsManager}
 */
public class DroolsManager implements IDroolsManager {

    // =========================================================================
    // Variables
    // =========================================================================

    /**
     * Logger
     */
    private static Log logger = LogFactory.getLog(DroolsManager.class);

    /**
     * The bundle context
     */
    private BundleContext bundleContext;

    /**
     * The working memory pool Service reference
     */
    private ServiceReference workingMemoryPoolReference;

    /**
     * The working memory Pool
     */
    private IDroolsWorkingMemoryPool droolsWorkingMemoryPool;

    /**
     * The managed working memory name
     */
    private String managedWorkingMemoryName;

    /**
     * The managed Drools working memory
     */
    private IDroolsWorkingMemory workingMemory;

    /**
     * The activated rule packages
     */
    private Map<String,ServiceRegistration> activatedRulePackages
        = new HashMap<String, ServiceRegistration>();

    /**
     * The persistence remote object
     */
    private RulePackageSLBRemote persistenceRemoteObject = null;

    /**
     * The JAXB context for rules packages serialization/deserialization
     * Must be declared with all the potentially involved classes.
     */
    private JAXBContext jaxbContext = JAXBContext.newInstance(TextualRulePackage.class,
															  VisualRulePackage.class,
															  Rule.class,
															  LHSCodeSnippet.class,
															  RHSCodeSnippet.class);
    
    
    // =========================================================================
    // Constructors
    // =========================================================================

    /**
     * Build a Drools working memory manager
     *
     * @throws Exception if it could not be created properly
     */
    public DroolsManager(final String managedWorkingMemoryName,
                         final BundleContext bundleContext) throws Exception {

        // Log a message
        logger.info("Creating a manager for the rule engine instance \"" + managedWorkingMemoryName +"\"");

        // Assign variables
        this.managedWorkingMemoryName = managedWorkingMemoryName;
        this.bundleContext = bundleContext;

        // Initialization
        try {

            // Retrieve the Drools Working Memory Pool
            retrieveWorkingMemoryPool();

            // Retrieve our WorkingMemory
            retrieveManagedWorkingMemory();

            // Retrieve the persistence remote object
            retrievePersistenceRemoteObject();

            // If no rules stored for this rule engine instance
            if (persistenceRemoteObject.countRulePackages(managedWorkingMemoryName) == 0) {

                // Log a message
                logger.info("Rule engine \"" + managedWorkingMemoryName +
                        "\" manager : No rule package stored, inserting the example rules");

                // Insert the example rule packages
                insertExamplePackages();
            }

            logger.info(persistenceRemoteObject.countRulePackages(managedWorkingMemoryName) +
                        " rule packages stored for \"" + managedWorkingMemoryName + "\"");

            // Activate rule packages that are marked as activated in the database
            IRulePackage[] packagesToActivate =
                persistenceRemoteObject.getRulePackages(null,
                                                        null,
                                                        null,
                                                        managedWorkingMemoryName,
                                                        null,
                                                        AbstractRulePackage.ACTIVATED,
                                                        null,
                                                        null);
            if (packagesToActivate != null) {
                for (IRulePackage pack : packagesToActivate) {

                    logger.info("Rule engine \"" + managedWorkingMemoryName +
                                "\" manager : Reactivating the rule package \"" +
                                pack.getDisplayName() + "\"");

                    activateRulePackage(pack);
                }
            }


        } catch (Exception e) {
            throw(new Exception("Could not build a manager for \"" +
                                managedWorkingMemoryName + "\"",e));
        }

        // Log a success message
        logger.info("Manager created for the rule engine instance \"" + managedWorkingMemoryName +"\"");
    }


    // =========================================================================
    // Implementation of IDroolsManager
    // =========================================================================

    // Rule engine control

    /**
     * Start the rule engine
     */
    public void start() throws Exception {
        logger.debug("");
        workingMemory.start();
    }

    /**
     * Stop the rule engine
     */
    public void stop() throws Exception {
        logger.debug("");
        workingMemory.stop();
    }

    /**
     * Get the rule engine state
     * @return a {@link DroolsWorkingMemoryState} describing the state
     * of the rule engine
     */
    public DroolsWorkingMemoryState getState() {
        return workingMemory.getState();
    }

    // Statistics

    /**
     * Get the name of all the working memories instantiated
     *
     * @return an array of strings
     */
    public List<String> getWorkingMemoriesNames() {
        return droolsWorkingMemoryPool.getWorkingMemoriesNames();
    }

    /**
     * Get the names of all the packages deployed
     * on the working memory
     *
     * @return the names as an array of strings
     */
    public List<String> getDeployedPackagesNames() {
        KnowledgeBase ruleBase = workingMemory.getKnowledgeBase();
        if (ruleBase == null) {
            return null;
        }
        ArrayList<String> res = new ArrayList<String>();
        Collection<KnowledgePackage> packages = ruleBase.getKnowledgePackages();
        for(KnowledgePackage p : packages){
            res.add(p.getName());
        }
        return res;
    }


    // Rules packages management

    /**
     * Update (or create) a rule package with the given version and apply
     * changes on the rule engine (activation, deactivation, update)
     */
    public RulePackageSaveReport commitRulePackageChanges(final IRulePackage rulePackage) throws Exception {
        logger.debug("");

        // Prepare to report the successes/failures to the client
        RulePackageSaveReport report = new RulePackageSaveReport();

        // Update the rule on the rule engine
        if (rulePackage.getState().equals(AbstractRulePackage.ACTIVATED)) {

            try {

                // Deactivate the old version of the rule package
                if (rulePackage.getPreviousSystemName() != null) {
                    deactivateRulePackage(rulePackage.getPreviousSystemName());
                } else {
                    deactivateRulePackage(rulePackage.getSystemName());
                }

                // Activate the new version of the rule package
                activateRulePackage(rulePackage);

                // Mark it as activated
                rulePackage.setState(AbstractRulePackage.ACTIVATED);

                // Log the operation for the client
                report.addSuccess("UPDATED");

            } catch (Exception e) {
                report.addFailure("The rule package was not updated.",
                                    "The rule engine is still running the old version" +
                                    " of the rule package. \n"
                                    + e.getMessage(),
                                    e.getStackTrace().toString());
            }

        } else if (rulePackage.getState().equals(AbstractRulePackage.ACTIVATION_REQUESTED)) {

            try {

                // Activate the rule package
                activateRulePackage(rulePackage);

                // Mark it as activated
                rulePackage.setState(AbstractRulePackage.ACTIVATED);

                // Log the operation for the client
                report.addSuccess(RulePackageSaveReport.OPERATION_ACTIVATE);

            } catch (Exception e) {
                report.addFailure("The rule package was not activated.",
                                    "The rule package could not be deployed.\n"
                                    + e.getMessage(),
                                    e.getStackTrace().toString());
            }

        } else if (rulePackage.getState().equals(AbstractRulePackage.DEACTIVATION_REQUESTED)) {

            try {

                // Deactivate the rule package
                if (rulePackage.getPreviousSystemName() != null) {
                    deactivateRulePackage(rulePackage.getPreviousSystemName());
                } else {
                    deactivateRulePackage(rulePackage.getSystemName());
                }

                // Mark it as deactivated
                rulePackage.setState(AbstractRulePackage.DEACTIVATED);

                // Log the operation for the client
                report.addSuccess(RulePackageSaveReport.OPERATION_DEACTIVATE);

            } catch (Exception e) {
                report.addFailure("The rule package was not deactivated.",
                                    "The rule engine is still running the rule package.\n"
                                    + e.getMessage(),
                                    e.getStackTrace().toString());
                e.printStackTrace();
            }
        }

        // Then save the rule package
        try {
            saveRulePackage(rulePackage);

            // Log the operation for the client
            report.addSuccess(RulePackageSaveReport.OPERATION_SAVE);

        } catch (Exception e) {
            e.printStackTrace();
            report.addFailure("The rule package was not saved",
                                e.getMessage(),
                                e.getStackTrace().toString());
        }

        // Return the report
        return report;
    }

    /**
     * Delete a rule package
     */
    public void deleteRulePackage(final IRulePackage rulePackage) throws Exception {
        logger.debug("");

        // If disconnected, try to connect to the database
        if(persistenceRemoteObject == null) {
            retrievePersistenceRemoteObject();
        }

        // If activated, deactivate the rule package on the engine
        if (rulePackage.getState().equals(AbstractRulePackage.ACTIVATED)) {

            // Deactivate the old version of the rule package
            if (rulePackage.getPreviousSystemName() != null) {
                deactivateRulePackage(rulePackage.getPreviousSystemName());
            } else {
                deactivateRulePackage(rulePackage.getSystemName());
            }
        }

        // Delete the database entry
        persistenceRemoteObject.deleteRulePackage(rulePackage);
    }

    /**
     * Check a rule package compilation
     */
    public List<RulePackageCompilationError> checkCompilation(final IRulePackage rulePackage) throws Exception {

        // If disconnected from the rule engine service, try to connect it
        if(workingMemory == null) {
            retrieveManagedWorkingMemory();
        }

        // Compile the package
        KnowledgeBuilder builder = workingMemory.newKnowledgeBuilder();
        Reader drlFileReader = new StringReader(rulePackage.getDrlSource());
        try {
            builder.add(ResourceFactory.newReaderResource(drlFileReader), ResourceType.DRL);
        } catch (Exception e) {
            // Catch the exception silently, we will treat the errors later
        }
        drlFileReader.close();

        // Build the result
        List<RulePackageCompilationError> res = new ArrayList<RulePackageCompilationError>();
        for (KnowledgeBuilderError error : builder.getErrors()) {
            res.add(new RulePackageCompilationError(error.getMessage()));
        }

        return res;
    }


    // Persistence

    /**
     * Get a rule package
     */
    public IRulePackage getRulePackage(String systemName) throws Exception {

        // If disconnected, try to connect to the database
        if(persistenceRemoteObject == null) {
            retrievePersistenceRemoteObject();
        }

        // Launch the request
        IRulePackage[] res = persistenceRemoteObject.getRulePackages(null,
                                                                       systemName,
                                                                     null,
                                                                       managedWorkingMemoryName,
                                                                     null,
                                                                     null,
                                                                     null,
                                                                     1);
        return res[0];
    }

    /**
     * Get all rule packages
     */
    public IRulePackage[] getRulePackages() throws Exception {

        // If disconnected, try to connect to the database
        if(persistenceRemoteObject == null) {
            retrievePersistenceRemoteObject();
        }

        // Launch the request
        return persistenceRemoteObject.getRulePackages(null,
                                                       null,
                                                       null,
                                                        managedWorkingMemoryName,
                                                       null,
                                                       null,
                                                       null,
                                                       1000);
    }

    /**
     * Save a rule package
     */
    public void saveRulePackage(IRulePackage rulePackage) throws Exception {
        logger.debug("");

        // If disconnected, try to connect to the database
        if(persistenceRemoteObject == null) {
            retrievePersistenceRemoteObject();
        }

        // Launch the request
        persistenceRemoteObject.saveRulePackage(rulePackage);
    }

    /**
     * Check a rule package name availability
     */
    public Boolean checkNameAvailability(String systemName) throws Exception {

        // If disconnected, try to connect to the database
        if(persistenceRemoteObject == null) {
            retrievePersistenceRemoteObject();
        }

        // Launch the request
        return persistenceRemoteObject.checkNameAvailability(systemName, managedWorkingMemoryName);
    }
    
    
    // Packages binding XML <=> Java
    
    /**
     * Serialize a rule package to XML
     */
    public String exportRulePackageToXML(IRulePackage rulePackage) throws Exception {
        logger.debug("");
    	
    	// Create a marshaller
    	Marshaller marshaller = jaxbContext.createMarshaller();
    	marshaller.setProperty("jaxb.formatted.output", true);
    	
    	// Serialize the rule package
    	StringWriter resWriter = new StringWriter();
    	if (rulePackage instanceof TextualRulePackage) {
    		marshaller.marshal((TextualRulePackage) rulePackage, resWriter);
    	} else if (rulePackage instanceof VisualRulePackage) {
    		marshaller.marshal((VisualRulePackage) rulePackage, resWriter);
    	}
    	return resWriter.toString();
    }
    
    /**
     * Build a rule package from its XML serialized representation
     * 
     * @param serializedRulePackage
     * @return
     * @throws Exception
     */
    public IRulePackage importRulePackageFromXML(String serializedRulePackage) throws Exception {
        logger.debug("");
    	
    	// Create an unmarshaller
    	Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
    	
    	// Deserialize the rule package
    	StringReader serializedRulePackageReader = new StringReader(serializedRulePackage);
    	return (IRulePackage) unmarshaller.unmarshal(serializedRulePackageReader);
    }

    
    // =========================================================================
    // Private methods
    // =========================================================================

    /**
     * Retrieve the Drools Working Memory Pool.
     *
     * @throws Exception
     */
    private void retrieveWorkingMemoryPool() throws Exception{

        // Retrieve working memory pool service reference
        try {
            workingMemoryPoolReference =
                bundleContext.getServiceReference(IDroolsWorkingMemoryPool.class.getName());
        } catch (Exception e) {
            throw(new Exception("Could not retrieve the working memory pool " +
                                "service reference",e));
        }

        // Retrieve working memory pool
        try {
            droolsWorkingMemoryPool =
                (IDroolsWorkingMemoryPool) bundleContext.getService(workingMemoryPoolReference);
        } catch (Exception e) {
            throw(new Exception("Could not retrieve working memory pool from" +
                                " the service reference",e));
        }

    }

    /**
     * Retrieve the managed Drools working memory
     *
     * @throws Exception
     */
    private void retrieveManagedWorkingMemory() throws Exception{
        try {
            workingMemory =
                droolsWorkingMemoryPool.getOrCreate(managedWorkingMemoryName);
        } catch (Exception e) {
            throw(new Exception("Could not retrieve the working memory " +
                    "\"" + managedWorkingMemoryName + "\"",e));
        }
    }

    /**
     * Retrieve the rules persistence remote object
     *
     * @throws Exception
     */
    private void retrievePersistenceRemoteObject() throws Exception {
        try {
            persistenceRemoteObject =
                (RulePackageSLBRemote)ServiceLocator.getInstance().getRemoteObject("db-ejb/rule");
        } catch (Exception e) {
            throw(new Exception("Could not connect to the rules database",e));
        }
    }

    /**
     * Activate a given rule package
     */
    private void activateRulePackage(final IRulePackage rulePackage) throws Exception {
        logger.debug("");

        // FIXME globals as a parameter
        Map<String, Object> globals = new HashMap<String, Object>();

        try {
            // Register the service
            ServiceRegistration packageAdderRegistration =
                    bundleContext.registerService(
                            IPackageAdder.class.getName(),
                            new DrlPackageAdder(bundleContext.getBundle(),
                                    globals,managedWorkingMemoryName,
                                    rulePackage.getSystemName(),
                                    rulePackage.getDrlSource()),
                            null);

            // Keep the service registration reference
            activatedRulePackages.put(rulePackage.getSystemName(), packageAdderRegistration);

        } catch (Exception e) {
            throw(new Exception("Could not activate the rule package",e));
        }

    }

    /**
     * Deactivate a rule package with the given name
     */
    private void deactivateRulePackage(final String rulePackageSystemName) throws Exception {
        logger.debug("");

        // Retrieve the service registration
        ServiceRegistration reg = activatedRulePackages.get(rulePackageSystemName);

        // Throw an exception if the rule package is not found
        if (reg == null) {
            throw new Exception("The package \"" + rulePackageSystemName + "\" was " +
                                " not found in the activated rule packages registry.");
        }

        try {
            // Try to unregister the service
            reg.unregister();

            // Remove it from the managed rules packages collection
            activatedRulePackages.remove(rulePackageSystemName);

        } catch (Exception e) {
            throw(new Exception("Could not deactivate the rule package",e));
        }
    }

    /**
     * Insert a set of example rule package in the database
     * @throws Exception
     */
    private void insertExamplePackages() throws Exception {
        logger.debug("");

        // Create example rule packages
        List<IRulePackage> exampleRulePackages = new ArrayList<IRulePackage>();

        TextualRulePackage rp1 = new TextualRulePackage();
        rp1.displayName = "Print Events";
        rp1.ruleEngineInstanceName = managedWorkingMemoryName;
        rp1.systemName = "org.ow2.jasmine.rules.packages.PrintEvents";
        rp1.description = "This is an example rule package with only one rule that matches" +
                          " every JasmineEventEB inserted" +
                          " and prints it in the console.";
        rp1.state = TextualRulePackage.DEACTIVATED;
        rp1.content =
            "/**" +
            "\n* JASMINe" +
            "\n* Copyright (C) 2009 Bull S.A.S." +
            "\n* Contact: jasmine@ow2.org" +
            "\n*" +
            "\n* This library is free software; you can redistribute it and/or" +
            "\n* modify it under the terms of the GNU Lesser General Public" +
            "\n* License as published by the Free Software Foundation; either" +
            "\n* version 2.1 of the License, or any later version." +
            "\n*" +
            "\n* This library is distributed in the hope that it will be useful," +
            "\n* but WITHOUT ANY WARRANTY; without even the implied warranty of" +
            "\n* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU" +
            "\n* Lesser General Public License for more details." +
            "\n*" +
            "\n* You should have received a copy of the GNU Lesser General Public" +
            "\n* License along with this library; if not, write to the Free Software" +
            "\n* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307" +
            "\n* USA" +
            "\n*/" +
            "\npackage org.ow2.jasmine.rules.packages.PrintEvents" +
            "\n" +
            "\nimport org.ow2.jasmine.event.beans.JasmineEventEB;" +
            "\n" +
            "\nrule \"Print each event\"" +
            "\n    no-loop false" +
            "\n    when" +
            "\n        $event : JasmineEventEB()" +
            "\n    then" +
            "\n           System.out.println(\"=============================================\");" +
            "\n           System.out.println(\"PrintEvents matched this event:              \");" +
            "\n           System.out.println($event.toString());" +
            "\n           System.out.println(\"=============================================\");" +
            "\n           drools.retract($event);" +
            "\nend";
        exampleRulePackages.add(rp1);

        TextualRulePackage rp2 = new TextualRulePackage();
        rp2.displayName = "Insert Jasmine Event In Notification Database";
        rp2.systemName = "org.ow2.jasmine.rules.packages.InsertJasmineEventInNotificationDatabase";
        rp2.ruleEngineInstanceName = managedWorkingMemoryName;
        rp2.description = "This is an example rule package with only one rule that matches" +
                          " every JasmineEventEB inserted and generates a notification" +
                          " inserted in the notifications database.";
        rp2.state = TextualRulePackage.DEACTIVATED;
        rp2.content =
            "/**" +
            "\n* JASMINe" +
            "\n* Copyright (C) 2009 Bull S.A.S." +
            "\n* Contact: jasmine@ow2.org" +
            "\n*" +
            "\n* This library is free software; you can redistribute it and/or" +
            "\n* modify it under the terms of the GNU Lesser General Public" +
            "\n* License as published by the Free Software Foundation; either" +
            "\n* version 2.1 of the License, or any later version." +
            "\n*" +
            "\n* This library is distributed in the hope that it will be useful," +
            "\n* but WITHOUT ANY WARRANTY; without even the implied warranty of" +
            "\n* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU" +
            "\n* Lesser General Public License for more details." +
            "\n*" +
            "\n* You should have received a copy of the GNU Lesser General Public" +
            "\n* License along with this library; if not, write to the Free Software" +
            "\n* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307" +
            "\n* USA" +
            "\n*/" +
            "\npackage org.ow2.jasmine.rules.packages.InsertJasmineEventInNotificationDatabase" +
            "\n" +
            "\nimport java.io.Serializable;" +
            "\nimport java.util.Date;" +
            "\n" +
            "\nimport org.objectweb.jasmine.rules.dispatcher.NotificationDispatcher;" +
            "\nimport org.ow2.jasmine.event.beans.JasmineEventEB;" +
            "\nimport org.ow2.jasmine.event.beans.JasmineEventNotification;" +
            "\n" +
            "\nrule \"Insert in database each event\"" +
            "\n    no-loop false" +
            "\n    when" +
            "\n        $event : JasmineEventEB()" +
            "\n    then" +
            "\n        // Create a new notification" +
            "\n        // notification id is auto-generated" +
            "\n        JasmineEventNotification notification = new JasmineEventNotification();" +
            "\n        " +
            "\n        // Set the notification message" +
            "\n        notification.setMessage($event.getProbe());" +
            "\n        // Set the notification date" +
            "\n        notification.setDate(new Date());" +
            "\n        // Set the notification level" +
            "\n        notification.setLevel(\"Info\");" +
            "\n        // set the notification status" +
            "\n        notification.setStatus(false);" +
            "\n        // Set the rule" +
            "\n        // The rules are configured in notification-config.xml file" +
            "\n        notification.setRule(\"InsertJasmineEventInNotificationDatabase\");" +
            "\n        " +
            "\n        // Add some details to the notification" +
            "\n        notification.addDetail(\"domain\", $event.getDomain());" +
            "\n        notification.addDetail(\"server\", $event.getServer());" +
            "\n        notification.addDetail(\"timestamp\", $event.getTimestamp().toString());" +
            "\n        " +
            "\n        // Send the notification to the dispatcher" +
            "\n        NotificationDispatcher.dispatchEvent(NotificationDispatcher.EventAction.SAVE_IN_DATABASE, notification);" +
            "\n		   " +
            "\n        drools.retract($event);" +
            "\nend";
        exampleRulePackages.add(rp2);

        TextualRulePackage rp21 = new TextualRulePackage();
        rp21.displayName = "Insert Jasmine Event In JMS Topic";
        rp21.systemName = "org.ow2.jasmine.rules.packages.InsertJasmineEventInJmsTopic";
        rp21.ruleEngineInstanceName = managedWorkingMemoryName;
        rp21.description = "This is an example rule package with only one rule that matches" +
                          " a JasmineEventEB on currentCpuLoad when value is too high and generates a notification" +
                          " inserted in the jms topic.";
        rp21.state = TextualRulePackage.DEACTIVATED;
        rp21.content =
            "/**" +
            "\n* JASMINe" +
            "\n* Copyright (C) 2009 Bull S.A.S." +
            "\n* Contact: jasmine@ow2.org" +
            "\n*" +
            "\n* This library is free software; you can redistribute it and/or" +
            "\n* modify it under the terms of the GNU Lesser General Public" +
            "\n* License as published by the Free Software Foundation; either" +
            "\n* version 2.1 of the License, or any later version." +
            "\n*" +
            "\n* This library is distributed in the hope that it will be useful," +
            "\n* but WITHOUT ANY WARRANTY; without even the implied warranty of" +
            "\n* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU" +
            "\n* Lesser General Public License for more details." +
            "\n*" +
            "\n* You should have received a copy of the GNU Lesser General Public" +
            "\n* License along with this library; if not, write to the Free Software" +
            "\n* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307" +
            "\n* USA" +
            "\n*/" +
            "\npackage org.ow2.jasmine.rules.packages.InsertJasmineEventInJmxTopic" +
            "\n" +
            "\nimport java.io.Serializable;" +
            "\nimport java.util.Date;" +
            "\n" +
            "\nimport org.objectweb.jasmine.rules.dispatcher.NotificationDispatcher;" +
            "\nimport org.ow2.jasmine.event.beans.JasmineEventEB;" +
            "\nimport org.ow2.jasmine.event.beans.JasmineEventNotification;" +
            "\n" +
            "\nrule \"Insert in database each event\"" +
            "\n    no-loop false" +
            "\n    when" +
            "\n        $event : JasmineEventEB(probe ==  \"java.lang:type=OperatingSystem:currentCpuLoad\", value > 20)" +
            "\n    then" +
            "\n        // Create a new notification" +
            "\n        // notification id is auto-generated" +
            "\n        JasmineEventNotification notification = new JasmineEventNotification();" +
            "\n        " +
            "\n        // Set the notification message" +
            "\n        notification.setMessage(\"Current CPU load is \" + $event.getValue());" +
            "\n        // Set the notification date" +
            "\n        notification.setDate($event.getTimestamp());" +
            "\n        // Set the notification level" +
            "\n        notification.setLevel(\"Info\");" +
            "\n        // set the notification status" +
            "\n        notification.setStatus(false);" +
            "\n        // Set the rule" +
            "\n        // The rules are configured in notification-config.xml file" +
            "\n        notification.setRule(\"InsertJasmineEventInNotificationDatabase\");" +
            "\n        " +
            "\n        // Add some details to the notification" +
            "\n        notification.addDetail(\"domain\", $event.getDomain());" +
            "\n        notification.addDetail(\"server\", $event.getServer());" +
            "\n        notification.addDetail(\"timestamp\", $event.getTimestamp().toString());" +
            "\n        " +
            "\n        // Send the notification to the dispatcher" +
            "\n        NotificationDispatcher.dispatchEvent(NotificationDispatcher.EventAction.JMS_TOPIC, notification);" +
            "\n		   " +
            "\n        drools.retract($event);" +
            "\nend";
        exampleRulePackages.add(rp21);

        TextualRulePackage rp3 = new TextualRulePackage();
        rp3.displayName = "Send Jasmine Event As Notification Mail";
        rp3.systemName = "org.ow2.jasmine.rules.packages.SendJasmineEventAsNotificationMail";
        rp3.ruleEngineInstanceName = managedWorkingMemoryName;
        rp3.description = "This is an example rule package with only one rule that matches" +
                          " every JasmineEventEB inserted and generates a mail notification.";
        rp3.state = TextualRulePackage.DEACTIVATED;
        rp3.content =
            "/**" +
            "\n* JASMINe" +
            "\n* Copyright (C) 2009 Bull S.A.S." +
            "\n* Contact: jasmine@ow2.org" +
            "\n*" +
            "\n* This library is free software; you can redistribute it and/or" +
            "\n* modify it under the terms of the GNU Lesser General Public" +
            "\n* License as published by the Free Software Foundation; either" +
            "\n* version 2.1 of the License, or any later version." +
            "\n*" +
            "\n* This library is distributed in the hope that it will be useful," +
            "\n* but WITHOUT ANY WARRANTY; without even the implied warranty of" +
            "\n* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU" +
            "\n* Lesser General Public License for more details." +
            "\n*" +
            "\n* You should have received a copy of the GNU Lesser General Public" +
            "\n* License along with this library; if not, write to the Free Software" +
            "\n* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307" +
            "\n* USA" +
            "\n*/" +
            "\npackage org.ow2.jasmine.rules.packages.SendJasmineEventAsNotificationMail" +
            "\n" +
            "\nimport java.io.Serializable;" +
            "\nimport java.util.Date;" +
            "\n" +
            "\nimport org.objectweb.jasmine.rules.dispatcher.NotificationDispatcher;" +
            "\nimport org.ow2.jasmine.event.beans.JasmineEventEB;" +
            "\nimport org.ow2.jasmine.event.beans.JasmineEventNotification;" +
            "\n" +
            "\nrule \"Send each event as a mail\"" +
            "\n    no-loop false" +
            "\n    when" +
            "\n        $event : JasmineEventEB()" +
            "\n    then" +
            "\n        // Create a new notification" +
            "\n        // notification id is auto-generated" +
            "\n        JasmineEventNotification notification = new JasmineEventNotification();" +
            "\n        " +
            "\n        // Set the notification message" +
            "\n        notification.setMessage($event.getProbe());" +
            "\n        // Set the notification date" +
            "\n        notification.setDate($event.getTimestamp());" +
            "\n        // Set the notification level" +
            "\n        notification.setLevel(\"Info\");" +
            "\n        // Set the notification status" +
            "\n        notification.setStatus(false);" +
            "\n        // Set the rule" +
            "\n        // The rules are configured in notification-config.xml file" +
            "\n        notification.setRule(\"SendJasmineEventAsNotificationMail\");" +
            "\n        " +
            "\n        // Add some details to the notification" +
            "\n        notification.addDetail(\"domain\", $event.getDomain());" +
            "\n        notification.addDetail(\"server\", $event.getServer());" +
            "\n        notification.addDetail(\"timestamp\", $event.getTimestamp().toString());" +
            "\n        " +
            "\n        // Send the notification to the dispatcher" +
            "\n        NotificationDispatcher.dispatchEvent(NotificationDispatcher.EventAction.SEND_MAIL, notification,\"jasmine@ow2.org\",\"to@address.com\");" +
            "\n" +
            "\nend";
        exampleRulePackages.add(rp3);

        TextualRulePackage rp4 = new TextualRulePackage();
        rp4.displayName = "Export Jasmine Event As JMX Notification";
        rp4.systemName = "org.ow2.jasmine.rules.packages.ExportJasmineEventAsJMXNotification";
        rp4.ruleEngineInstanceName = managedWorkingMemoryName;
        rp4.description = "Exports each matched JASMINe event as a JMX notification.";
        rp4.state = TextualRulePackage.DEACTIVATED;
        rp4.content =
            "/**" +
            "\n* JASMINe" +
            "\n* Copyright (C) 2009 Bull S.A.S." +
            "\n* Contact: jasmine@ow2.org" +
            "\n*" +
            "\n* This library is free software; you can redistribute it and/or" +
            "\n* modify it under the terms of the GNU Lesser General Public" +
            "\n* License as published by the Free Software Foundation; either" +
            "\n* version 2.1 of the License, or any later version." +
            "\n*" +
            "\n* This library is distributed in the hope that it will be useful," +
            "\n* but WITHOUT ANY WARRANTY; without even the implied warranty of" +
            "\n* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU" +
            "\n* Lesser General Public License for more details." +
            "\n*" +
            "\n* You should have received a copy of the GNU Lesser General Public" +
            "\n* License along with this library; if not, write to the Free Software" +
            "\n* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307" +
            "\n* USA" +
            "\n*/" +
            "\npackage org.ow2.jasmine.rules.packages.ExportJasmineEventAsJMXNotification" +
            "\n" +
            "\nimport java.io.Serializable;" +
            "\nimport java.util.Date;" +
            "\n" +
            "\nimport org.objectweb.jasmine.rules.dispatcher.NotificationDispatcher;" +
            "\nimport org.ow2.jasmine.event.beans.JasmineEventEB;" +
            "\nimport org.ow2.jasmine.event.beans.JasmineEventNotification;" +
            "\n" +
            "\nrule \"export each event as a JMX notification\"" +
            "\n    no-loop false" +
            "\n    when" +
            "\n        $event : JasmineEventEB()" +
            "\n    then" +
            "\n        // Create a new notification" +
            "\n        // notification id is auto-generated" +
            "\n        JasmineEventNotification notification = new JasmineEventNotification();" +
            "\n        " +
            "\n        // Set the notification message" +
            "\n        notification.setMessage($event.getProbe());" +
            "\n        // Set the notification date" +
            "\n        notification.setDate($event.getTimestamp());" +
            "\n        // Set the notification level" +
            "\n        notification.setLevel(\"Info\");" +
            "\n        // Set the notification status" +
            "\n        notification.setStatus(false);" +
            "\n        // Set the rule" +
            "\n        // The rules are configured in notification-config.xml file" +
            "\n        notification.setRule(\"ExportJasmineEventAsJMXNotification\");" +
            "\n        " +
            "\n        // Add some details to the notification" +
            "\n        notification.addDetail(\"domain\", $event.getDomain());" +
            "\n        notification.addDetail(\"server\", $event.getServer());" +
            "\n        notification.addDetail(\"timestamp\", $event.getTimestamp().toString());" +
            "\n        " +
            "\n        // Send the notification to the dispatcher" +
            "\n        NotificationDispatcher.dispatchEvent(NotificationDispatcher.EventAction.JMX_EXPORT, notification);" +
            "\n" +
            "\nend";

        exampleRulePackages.add(rp4);

        // Save the packages
        for (IRulePackage rulePackage : exampleRulePackages) {
            saveRulePackage(rulePackage);
        }

    }

}
