/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.manager.service;

import java.util.List;

import org.ow2.jasmine.rules.api.DroolsWorkingMemoryState;
import org.ow2.jasmine.rules.manager.model.IRulePackage;

/**
 * Manage a given Drools working memory
 *
 * @author Julien Nicoulaud
 * @see {@link DroolsManager}
 */
public interface IDroolsManager {

    // Rule engine control

    /**
     * Start the rule engine
     */
    public void start() throws Exception;

    /**
     * Stop the rule engine
     */
    public void stop() throws Exception;

    /**
     * Get the rule engine state
     * @return a {@link DroolsWorkingMemoryState} describing the state
     * of the rule engine
     */
    public DroolsWorkingMemoryState getState();


    // Statistics

    /**
     * Get the names of all the working memories instantiated
     *
     * @return the names as an array of strings
     */
    public List<String> getWorkingMemoriesNames();

    /**
     * Get the names of all the packages deployed
     * on the working memory
     *
     * @return the names as an array of strings
     */
    public List<String> getDeployedPackagesNames();


    // Rules packages management
    
    /**
     * Update (or create) a rule package with the given version and apply
     * changes on the rule engine (activation, deactivation, update)
     */
    public RulePackageSaveReport commitRulePackageChanges(final IRulePackage rulePackage) throws Exception;

    /**
     * Delete a rule package
     */
    public void deleteRulePackage(final IRulePackage rulePackage) throws Exception;
    
    /**
     * Check a rule package compilation
     */
    public List<RulePackageCompilationError> checkCompilation(final IRulePackage rulePackage) throws Exception;
    
    
    // Persistence
    
    /**
     * Get all rule packages
     */
    public IRulePackage[] getRulePackages() throws Exception;
    
    /**
     * Get a rule package
     */
    public IRulePackage getRulePackage(String systemName) throws Exception;
    
    /**
     * Save a rule package
     */
    public void saveRulePackage(IRulePackage rulePackage) throws Exception;
    
    /**
     * Check a rule package name availability
     */
    public Boolean checkNameAvailability(String systemName) throws Exception;
    
    
    // Packages binding XML <=> Java
    
    /**
     * Serialize a rule package to XML
     */
    public String exportRulePackageToXML(IRulePackage rulePackage) throws Exception;
    
    /**
     * Build a rule package from its XML serialized representation
     * 
     * @param serializedRulePackage
     * @return
     * @throws Exception
     */
    public IRulePackage importRulePackageFromXML(String serializedRulePackage) throws Exception;
}
