/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.cluster.cmi.ejb;

import java.net.InetAddress;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.EJBObject;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.management.remote.JMXServiceURL;

import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.ow2.cmi.controller.common.ClusterViewManagerException;
import org.ow2.cmi.controller.server.ProtocolNotFoundException;
import org.ow2.cmi.controller.server.ServerClusterViewManager;
import org.ow2.cmi.controller.server.ServerClusterViewManagerException;
import org.ow2.cmi.ha.SessionId;
import org.ow2.cmi.info.ClusteredObjectInfo;
import org.ow2.cmi.lb.policy.IPolicy;
import org.ow2.cmi.lb.strategy.IStrategy;
import org.ow2.cmi.reference.CMIReference;
import org.ow2.cmi.reference.ObjectNotFoundException;
import org.ow2.cmi.reference.ServerId;
import org.ow2.cmi.reference.ServerNotFoundException;
import org.ow2.cmi.reference.ServerRef;
import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.util.pool.api.IPoolConfiguration;

/**
 * A bean to configure CMI, without its MBean.
 * @author Loris Bouzonnet
 */
@Stateless
@Remote(IEJBServerClusterViewManager.class)
public class EJBServerClusterViewManager implements IEJBServerClusterViewManager {

    /**
     * Bundle context that will be injected by the container.
     */
    @OSGiResource
    private BundleContext bundleContext = null;

    private ServerClusterViewManager clusterViewManager;

    private ServiceReference clusterViewManagerReference;

    private ServiceListener clusterViewManagerListener;

    /**
     * Bean startup call-back method.
     */
    @PostConstruct
    public void start() {
        clusterViewManagerListener = new ServiceListener() {
            public void serviceChanged(final ServiceEvent event) {
                ServiceReference ref = event.getServiceReference();
                // React on REGISTERED|UNREGISTERING events
                switch(event.getType()) {
                case ServiceEvent.REGISTERED:
                    clusterViewManager = (ServerClusterViewManager) bundleContext.getService(ref);
                    break;
                case ServiceEvent.UNREGISTERING:
                    clusterViewManager = null;
                    bundleContext.ungetService(ref);
                    break;
                }
            }
        };
        try {
            bundleContext.addServiceListener(clusterViewManagerListener,
                    "(objectClass=" + ServerClusterViewManager.class.getName() + ")");
        } catch (InvalidSyntaxException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        clusterViewManagerReference = bundleContext.getServiceReference(
                ServerClusterViewManager.class.getName());
        clusterViewManager = (ServerClusterViewManager) bundleContext.getService(
                clusterViewManagerReference);
    }

    /**
     * Bean stop call-back method.
     */
    @PreDestroy
    public void stop() {
        if(clusterViewManagerListener != null) {
            bundleContext.removeServiceListener(clusterViewManagerListener);
        }
        if(clusterViewManagerReference != null) {
            bundleContext.ungetService(clusterViewManagerReference);
        }
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#addObjectInstance(org.ow2.cmi.info.ClusteredObjectInfo, org.ow2.cmi.reference.CMIReference)
     */
    public void addObjectInstance(final ClusteredObjectInfo clusteredObjectInfo,
            final CMIReference cmiReference) throws ServerClusterViewManagerException {
        clusterViewManager.addObjectInstance(clusteredObjectInfo, cmiReference);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#addPoolToEmpty(java.lang.String)
     */
    public void addPoolToEmpty(final String objectName) {
        clusterViewManager.addPoolToEmpty(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#addServerToBlackList(org.ow2.cmi.reference.ServerRef)
     */
    public void addServerToBlackList(final ServerRef serverRef) {
        clusterViewManager.addServerToBlackList(serverRef);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getAvailablePoliciesAndStrategies()
     */
    public Map<String, Set<String>> getAvailablePoliciesAndStrategies() {
        return clusterViewManager.getAvailablePoliciesAndStrategies();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getBusinessName(java.lang.String)
     */
    public String getBusinessName(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getBusinessName(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getCMIReferences(java.lang.String)
     */
    public Collection<CMIReference> getCMIReferences(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getCMIReferences(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getClusterNames()
     */
    public Set<String> getClusterNames() {
        return clusterViewManager.getClusterNames();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getInetAddress()
     */
    public InetAddress getInetAddress() {
        return clusterViewManager.getInetAddress();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getItfName(java.lang.String)
     */
    public String getItfName(final String objectName) throws ObjectNotFoundException {
        return clusterViewManager.getItfName(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getJMXServiceURL(java.lang.String)
     */
    public JMXServiceURL getJMXServiceURL(final String protocolName) throws ProtocolNotFoundException {
        return clusterViewManager.getJMXServiceURL(protocolName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getNbClientsConnectedToProvider()
     */
    public int getNbClientsConnectedToProvider() {
        return clusterViewManager.getNbClientsConnectedToProvider();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getObjectNames()
     */
    public Set<String> getObjectNames() {
        return clusterViewManager.getObjectNames();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getObjectNames(java.lang.String)
     */
    public Set<String> getObjectNames(final String clusterName) {
        return clusterViewManager.getObjectNames(clusterName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getProviderURLs(java.lang.String)
     */
    public List<String> getProviderURLs(final String protocolName)
            throws ServerClusterViewManagerException {
        return clusterViewManager.getProviderURLs(protocolName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getRefOnLocalRegistry(java.lang.String)
     */
    public ServerRef getRefOnLocalRegistry(final String protocolName) throws ProtocolNotFoundException {
        return clusterViewManager.getRefOnLocalRegistry(protocolName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getServerObjectForProtocol(java.lang.String, java.lang.String)
     */
    public Set<String> getServerObjectForProtocol(final String serverUrl,
            final String protocolName) throws ProtocolNotFoundException {
        return clusterViewManager.getServerObjectForProtocol(serverUrl, protocolName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getServerRefs(java.lang.String)
     */
    public Set<ServerId> getServerRefs(final String protocolName)
            throws ProtocolNotFoundException {
        return clusterViewManager.getServerRefs(protocolName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#isReplicationManagerStarted()
     */
    public boolean isReplicationManagerStarted() {
        return clusterViewManager.isReplicationManagerStarted();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#isServerBlackListed(org.ow2.cmi.reference.ServerRef)
     */
    public boolean isServerBlackListed(final ServerRef serverRef) {
        return clusterViewManager.isServerBlackListed(serverRef);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#removeCMIReference(org.ow2.cmi.reference.CMIReference)
     */
    public void removeCMIReference(final CMIReference cmiReference) {
        clusterViewManager.removeCMIReference(cmiReference);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#removePoolToEmpty(java.lang.String)
     */
    public void removePoolToEmpty(final String objectName) {
        clusterViewManager.removePoolToEmpty(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#removeServerFromBlackList(org.ow2.cmi.reference.ServerRef)
     */
    public void removeServerFromBlackList(final ServerRef serverRef) {
        clusterViewManager.removeServerFromBlackList(serverRef);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#setAlgorithmForPolicy(java.lang.String, java.lang.String, java.lang.String, java.util.Map)
     */
    public void setAlgorithmForPolicy(final String objectName,
            final String policyClassName, final String strategyClassName,
            final Map<String, Object> properties) throws ObjectNotFoundException {
        clusterViewManager.setAlgorithmForPolicy(objectName, policyClassName, strategyClassName, properties);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#setDelayToRefresh(int)
     */
    public void setDelayToRefresh(final int delay) {
        clusterViewManager.setDelayToRefresh(delay);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#setLoadFactor(org.ow2.cmi.reference.ServerRef, int)
     */
    public void setLoadFactor(final ServerRef serverRef, final int loadFactor) {
        clusterViewManager.setLoadFactor(serverRef, loadFactor);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#setPolicyClassName(java.lang.String, java.lang.String)
     */
    public void setPolicyClassName(final String objectName, final String policyClassName)
            throws ObjectNotFoundException {
        clusterViewManager.setPolicyClassName(objectName, policyClassName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#setPoolConfiguration(java.lang.String, org.ow2.util.pool.api.IPoolConfiguration)
     */
    public void setPoolConfiguration(final String objectName,
            final IPoolConfiguration poolConfiguration)
            throws ObjectNotFoundException {
        clusterViewManager.setPoolConfiguration(objectName, poolConfiguration);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#setPropertiesForPolicy(java.lang.String, java.util.Map)
     */
    public void setPropertiesForPolicy(final String objectName,
            final Map<String, Object> properties) throws ObjectNotFoundException {
        clusterViewManager.setPropertiesForPolicy(objectName, properties);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#setPropertyForPolicy(java.lang.String, java.lang.String, java.lang.Object)
     */
    public void setPropertyForPolicy(final String objectName, final String propertyName,
            final Object propertyValue) throws ObjectNotFoundException {
        clusterViewManager.setPropertyForPolicy(objectName, propertyName, propertyValue);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#setStrategyClassName(java.lang.String, java.lang.String)
     */
    public void setStrategyClassName(final String objectName, final String strategyClassName)
            throws ObjectNotFoundException {
        clusterViewManager.setStrategyClassName(objectName, strategyClassName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getApplicationExceptionNames(java.lang.String)
     */
    public Set<String> getApplicationExceptionNames(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getApplicationExceptionNames(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getCMIReferences(java.lang.String, java.lang.String)
     */
    public Collection<CMIReference> getCMIReferences(final String objectName,
            final String protocolName) throws ObjectNotFoundException {
        return clusterViewManager.getCMIReferences(objectName, protocolName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getClusterName(java.lang.String)
     */
    public String getClusterName(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getClusterName(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getDateOfProperties(java.lang.String)
     */
    public long getDateOfProperties(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getDateOfProperties(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getDelayToRefresh()
     */
    public int getDelayToRefresh() {
        return clusterViewManager.getDelayToRefresh();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getInitialContextFactoryName(java.lang.String)
     */
    public String getInitialContextFactoryName(final String protocolName) throws ProtocolNotFoundException {
        return clusterViewManager.getInitialContextFactoryName(protocolName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getInterface(java.lang.String)
     */
    public Class<?> getInterface(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getInterface(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getLoadFactor(org.ow2.cmi.reference.ServerRef)
     */
    public int getLoadFactor(final ServerRef serverRef)
            throws ServerNotFoundException {
        return clusterViewManager.getLoadFactor(serverRef);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getPolicy(java.lang.String)
     */
    public IPolicy<CMIReference> getPolicy(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getPolicy(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getPolicyClass(java.lang.String)
     */
    public Class<? extends IPolicy<?>> getPolicyClass(final String objectName)
            throws ObjectNotFoundException, ClusterViewManagerException {
        return clusterViewManager.getPolicyClass(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getPolicyClassName(java.lang.String)
     */
    public String getPolicyClassName(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getPolicyClassName(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getPoolConfiguration(java.lang.String)
     */
    public IPoolConfiguration getPoolConfiguration(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getPoolConfiguration(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getPropertiesForPolicy(java.lang.String)
     */
    public Map<String, Object> getPropertiesForPolicy(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getPropertiesForPolicy(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getPropertyForPolicy(java.lang.String, java.lang.String)
     */
    public Object getPropertyForPolicy(final String objectName, final String propertyName)
            throws ObjectNotFoundException {
        return clusterViewManager.getPropertyForPolicy(objectName, propertyName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getProtocols()
     */
    public Set<String> getProtocols() {
        return clusterViewManager.getProtocols();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getRemoteClass(java.lang.String)
     */
    public Class<? extends EJBObject> getRemoteClass(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getRemoteClass(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getSessionId()
     */
    public SessionId getSessionId() {
        return clusterViewManager.getSessionId();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getStrategyClass(java.lang.String)
     */
    public Class<? extends IStrategy<?>> getStrategyClass(final String objectName)
            throws ObjectNotFoundException, ClusterViewManagerException {
        return clusterViewManager.getStrategyClass(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getStrategyClassName(java.lang.String)
     */
    public String getStrategyClassName(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.getStrategyClassName(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#getUUID()
     */
    public UUID getUUID() {
        return clusterViewManager.getUUID();
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#hasState(java.lang.String)
     */
    public boolean hasState(final String objectName) throws ObjectNotFoundException {
        return clusterViewManager.hasState(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#isClustered(java.lang.String)
     */
    public boolean isClustered(final String objectName) {
        return clusterViewManager.isClustered(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#isPoolToEmpty(java.lang.String)
     */
    public boolean isPoolToEmpty(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.isPoolToEmpty(objectName);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.cluster.IEJBServerClusterViewManager#isReplicated(java.lang.String)
     */
    public boolean isReplicated(final String objectName)
            throws ObjectNotFoundException {
        return clusterViewManager.isReplicated(objectName);
    }

}
